<?php

namespace Waylet\Payment\Helper;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Framework\Serialize\Serializer\Json;
use Psr\Log\LoggerInterface;

class Api
{
    const API_ENDPOINT_PRE = 'https://pre.wayletlabs.com/api/openwaylet/web/v1/transactions';
    const API_ENDPOINT_PRO = 'https://pro.waylet.es/api/openwaylet/web/v1/transactions';
    const API_ENDPOINT_INTEGRATIONS_PRE = 'https://pre.wayletlabs.com/api/integrations/openwaylet/v2/payments';
    const API_ENDPOINT_INTEGRATIONS_PRO = 'https://pro.wayletlabs.com/api/integrations/openwaylet/v2/payments';

    /**
     * @var Curl
     */
    private $curlClient;

    /**
     * @var Json
     */
    private $jsonSerializer;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    private EncryptorInterface $encryptor;

    /**
     * Api constructor.
     *
     * @param Curl $curlClient
     * @param Json $jsonSerializer
     * @param LoggerInterface $logger
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        Curl $curlClient,
        Json $jsonSerializer,
        LoggerInterface $logger,
        EncryptorInterface $encryptor
    ) {
        $this->curlClient = $curlClient;
        $this->jsonSerializer = $jsonSerializer;
        $this->logger = $logger;
        $this->scopeConfig = $scopeConfig;
        $this->encryptor = $encryptor;
    }

    private function getConfig($config)
    {
        return $this->scopeConfig->getValue("payment/waylet_payment/{$config}");
    }

    private function getApiKey() : ?string
    {
        return $this->encryptor->decrypt($this->getConfig('api_key'));
    }

    private function getSecret() : ?string
    {
        return $this->encryptor->decrypt($this->getConfig('secret'));
    }

    private function getEndpoint() : string
    {
        return $this->getConfig('debug') == 1 ? self::API_ENDPOINT_PRE : self::API_ENDPOINT_PRO;
    }

    private function getIntegrationsEndpoint() : string
    {
        return $this->getConfig('debug') == 1 ? self::API_ENDPOINT_INTEGRATIONS_PRE : self::API_ENDPOINT_INTEGRATIONS_PRO;
    }

    private function getCommerceId()
    {
        return $this->getConfig('commerce_id');
    }
    /**
     * Call API to start payment
     *
     * @param array $requestData
     * @return array|null
     */
    public function startPayment(array $requestData)
    {   /*
        $apiKey = "apikey"; // API Key provided by Waylet
        $customer = $requestData['customer'];
        $amount = $requestData['amount'];
        $commerceId = $requestData['commerceId'];
        $callbackUrl = $requestData['callbackUrl'];
        $paymentId = $requestData['paymentId'];

        // Create signature
        $signature = hash_hmac('sha512', $customer . $amount . $commerceId . $paymentId . $callbackUrl, $secret);

        // Add signature to request data
        $requestData['signature'] = $signature;

        try {
            $headers = new Header();
            $headers->set('x-openwaylet-apikey', $apiKey);
            $this->curlClient->setHeaders($headers);
            $this->curlClient->post(self::API_ENDPOINT_PRE . '/start', $requestData);
            $responseBody = $this->curlClient->getBody();

            // Process response body as needed
            $response = $this->jsonSerializer->unserialize($responseBody);

            return $response;
        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());
            return null;
        }*/
    }

    /**
     * Call API to cancel payment
     *
     * @param array $requestData
     * @return array|null
     */
    public function cancelPayment(array $requestData)
    {/*
        try {
            // Calculate signature
            $signature = hash_hmac(
                'sha512',
                $requestData['commerceId'] . $requestData['paymentId'],
                $this->apiPassword
            );

            // Include signature in request data
            $requestData['signature'] = $signature;

            // Make API call
            $this->curlClient->post(self::API_ENDPOINT_PRE . '/cancel', $requestData);
            $responseBody = $this->curlClient->getBody();

            // Process response body as needed
            $response = $this->jsonSerializer->unserialize($responseBody);

            return $response;
        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());
            return null;
        }*/
    }

    /**
     * Call API to refund payment
     *
     * @param array $requestData
     * @return array|null
     */
    public function refundPayment(array $requestData)
    {

        $requestData['amount'] = 100 * $requestData['amount']; 

        // Calculate signature
        $signature = hash_hmac('sha512', $requestData['amount'] . $this->getCommerceId() . $requestData['paymentId'], $this->getSecret());

        // Build headers
        $headers = [
            'Content-Type'        => 'application/json',
            'x-openwaylet-apikey' => $this->getApiKey()
        ];

        $requestData['commerceId'] = $this->getCommerceId();
        $requestData['signature'] = $signature;

        // Build request data
        $requestData = json_encode($requestData);

        try {
            $this->curlClient->setHeaders($headers);
            $this->logger->debug('Refund Waylet request' . json_encode($requestData));
            $this->curlClient->post($this->getIntegrationsEndpoint() . '/refund', $requestData);
            $responseBody = $this->curlClient->getBody();
            $this->logger->debug('Refund Waylet response:' . $responseBody);
            // Process response body as needed
            $response = $this->jsonSerializer->unserialize($responseBody);

            return $response;
        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());
            return null;
        }
    }

    /**
     * Request payment URL
     *
     * @param array $postData
     * @return string|null
     */
    public function requestPaymentUrl(array $postData, $isDebug, $apiKey)
    {
        try {
            $url = self::API_ENDPOINT_PRO;
            if ($isDebug) {;
                $url = self::API_ENDPOINT_PRE;
            }
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($postData));
            curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json', 'x-openwaylet-apikey:' . $apiKey));
            $response = curl_exec($ch);
            curl_close($ch);

            return $response;
        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());
            return null;
        }
    }

    /**
     * Verifica si la solicitud enviada por Waylet contiene todos los campos requeridos y
     * si la firma de la solicitud coincide con la firma esperada.
     *
     * @param array $requestData Los datos enviados por Waylet.
     * @param string $secret La clave secreta para generar la firma esperada.
     * @return bool Retorna `true` si la firma de la solicitud coincide con la firma esperada,
     * o `false` en caso contrario.
     */
    public function verifyHas(array $requestData, $secret)
    {
        $requiredFields = ['paymentId', 'wayletPaymentId', 'status', 'amount', 'signature'];
        $hasAllFields = true;

        // Check if all required fields are present
        foreach ($requiredFields as $field) {
            if (!isset($requestData[$field])) {
                $hasAllFields = false;
                break;
            }
        }

        if (!$hasAllFields) {
            return false;
        }

        // Convert input data to expected types
        $paymentId = (string) $requestData['paymentId'];
        $wayletPaymentId = (string) $requestData['wayletPaymentId'];
        $status = (string) trim($requestData['status']);
        $amount = (string) $requestData['amount'];
        $signature = (string) $requestData['signature'];

        // Create signature
        $localHash = hash_hmac('sha512', $paymentId . $wayletPaymentId . $status . $amount, $secret);

        // Compare signatures in a timing-attack safe way
        $isSignatureValid = hash_equals($localHash, $signature);

        return $isSignatureValid;
    }
}
